#!/usr/bin/env python
import json
import argparse
from collections import OrderedDict
from tripal import TripalAuth, TripalInstance

class index(object):

    def run(self, args):
        parser = argparse.ArgumentParser(prog=('tripal %s' % self.__class__.__name__), description='Index data with elasticsearch')
        TripalAuth(parser)
        parser.add_argument('-t', '--table', default='index_website', help='Name of a specific table to index (default="index_website" to index all the tripal content)')
        parser.add_argument('-q', '--queues', type=int, default=10, help='Number of cron queues to use')
        parser.add_argument('-f', '--fields', nargs='*', default=[], help='List of column names to index (omit to index all columns)')
        parser.add_argument('-l', '--links', nargs='*', default=[], help='List of links to show to users, syntax: <column-where-to-show-the-link>|</your/url/[any-column-name]>')

        args = parser.parse_args(args)

        links = {}
        for l in args.links:
            ls = l.split('|', 1)
            if ls[0] not in args.fields:
                raise Exception("Cannot add a link to column '%s' because it is not indexed" % ls[0])
            links[ls[0]] = ls[1]

        ti = TripalInstance(**vars(args))

        res = ti.tripaldb.index(args.table, args.queues, args.fields, links)

        print 'Scheduled indexing with elastic search'



#!/usr/bin/env python
import os
import json
import argparse
from tripal import TripalAuth, TripalAnalysis, TripalInstance

class load_interpro(object):

    def run(self, args):
        parser = argparse.ArgumentParser(prog=('tripal %s' % self.__class__.__name__), description='Loads InterPro results into Tripal (requires tripal_analysis_interpro module)')
        TripalAuth(parser)
        TripalAnalysis(parser)
        parser.add_argument('interpro', help='Path to the InterProScan file to load (single XML file, or directory containing multiple XML files)')
        parser.add_argument('--interpro-parameters', help='InterProScan parameters used to produce these results')
        parser.add_argument('--parse-go', action='store_true', help='Load GO annotation to the database')
        parser.add_argument('--query-re', help='The regular expression that can uniquely identify the query name. This parameters is required if the feature name does not identically match the query name.')
        parser.add_argument('--query-type', help='The feature type (e.g. \'gene\', \'mRNA\', \'contig\') of the query. It must be a valid Sequence Ontology term.')
        parser.add_argument('--query-uniquename', action='store_true', help='Use this if the --query-re regular expression matches unique names instead of names in the database.')

        args = parser.parse_args(args)

        ti = TripalInstance(**vars(args))

        params = ti.analysis.getBasePayload(args)

        params.update({
            'type': 'chado_analysis_interpro',
            'interprofile': args.interpro,
            'interprojob': 1,
            'parsego': int(args.parse_go),
            'interproparameters': args.interpro_parameters,
            'query_re': args.query_re,
            'query_type': args.query_type,
            'query_uniquename': args.query_uniquename,
        })

        res = ti.analysis.addAnalysis(params)

        print "New Interpro analysis created with ID: %s" % res['nid']



#!/usr/bin/env python
import json
import argparse
from collections import OrderedDict
from tripal import TripalAuth, TripalInstance

class sync_analysis(object):

    def run(self, args):
        parser = argparse.ArgumentParser(prog=('tripal %s' % self.__class__.__name__), description='Synchronize analysis')
        TripalAuth(parser)
        parser.add_argument('--job-name', help='Name of the job (default=\'Sync Analysis\')')
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument('--analysis', help='Analysis name')
        group.add_argument('--analysis-id', help='Analysis ID')

        args = parser.parse_args(args)

        ti = TripalInstance(**vars(args))

        an_id = None
        if args.analysis:
            an_id = ti.analysis.getAnalysisByName(args.analysis)['analysis_id']
        elif args.analysis_id:
            an_id = args.analysis_id
        else:
            raise Exception("Either --analysis or --analysis-id is required")

        job_name = args.job_name
        if not job_name:
            job_name = 'Sync Analysis'

        transaction = 1 # use transaction or not, no reason to disable this

        job_args = OrderedDict()
        job_args['base_table'] = 'analysis'
        job_args['max_sync'] = ''
        job_args['organism_id'] = ''
        job_args['types'] = []
        job_args['ids'] = [int(an_id)]
        job_args['linking_table'] = 'chado_analysis'
        job_args['node_type'] = 'chado_analysis'

        r = ti.jobs.addJob(job_name, 'chado_feature', 'chado_node_sync_records', job_args)
        print 'Sync analysis job scheduled with id %s' % r['job_id']



#!/usr/bin/env python
import os
import json
import argparse
from tripal import TripalAuth, TripalAnalysis, TripalInstance

class load_blast(object):

    def run(self, args):
        parser = argparse.ArgumentParser(prog=('tripal %s' % self.__class__.__name__), description='Loads Blast results into Tripal (requires tripal_analysis_blast module)')
        TripalAuth(parser)
        TripalAnalysis(parser)
        parser.add_argument('blast', help='Path to the Blast file to load (single XML file, or directory containing multiple XML files)')
        parser.add_argument('--blast-ext', help='If looking for files in a directory, extension of the blast result files')
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument('--blastdb', help='Name of the database blasted against (must be in the Chado db table)')
        group.add_argument('--blastdb-id', help='ID of the database blasted against (must be in the Chado db table)')
        parser.add_argument('--blast-parameters', help='Blast parameters used to produce these results')
        parser.add_argument('--max-hits', type=int, default=25, help='Maximum number of hits kept (default=25)')
        parser.add_argument('--query-re', help='The regular expression that can uniquely identify the query name. This parameters is required if the feature name is not the first word in the blast query name.')
        parser.add_argument('--query-type', help='The feature type (e.g. \'gene\', \'mRNA\', \'contig\') of the query. It must be a valid Sequence Ontology term.')
        parser.add_argument('--query-uniquename', action='store_true', help='Use this if the --query-re regular expression matches unique names instead of names in the database.')
        parser.add_argument('--is-concat', action='store_true', help='If the blast result file is simply a list of concatenated blast results.')
        parser.add_argument('--search-keywords', action='store_true', help='Extract keywords for Tripal search')

        args = parser.parse_args(args)

        ti = TripalInstance(**vars(args))

        blastdb_id = None
        if args.blastdb:
            blastdb_id = ti.db.getDbByName(args.blastdb)['db_id']
        elif args.blastdb_id:
            blastdb_id = args.blastdb_id
        else:
            raise Exception("Either --blastdb or --blastdb-id is required")

        params = ti.analysis.getBasePayload(args)

        params.update({
            'type': 'chado_analysis_blast',
            'blastdb': blastdb_id,
            'blastfile': args.blast,
            'blastfile_ext': args.blast_ext,
            'blastjob': 1, # no reason to not launch a job
            'blastparameters': args.blast_parameters,
            'query_re': args.query_re,
            'query_type': args.query_type,
            'query_uniquename': args.query_uniquename,
            'is_concat': int(args.is_concat),
            'search_keywords': int(args.search_keywords),
        })

        res = ti.analysis.addAnalysis(params)

        print "New Blast analysis created with ID: %s" % res['nid']



#!/usr/bin/env python
import os
import json
import argparse
from tripal import TripalAuth, TripalAnalysis, TripalInstance

class load_go(object):

    def run(self, args):
        parser = argparse.ArgumentParser(prog=('tripal %s' % self.__class__.__name__), description='Loads Blast2GO results into Tripal (requires tripal_analysis_go module)')
        TripalAuth(parser)
        TripalAnalysis(parser)
        parser.add_argument('gaf', help='Path to the GAF file to load (single XML file, or directory containing multiple XML files)')
        parser.add_argument('--gaf-ext', help='If looking for files in a directory, extension of the GAF files')
        parser.add_argument('--query-type', help='The feature type (e.g. \'gene\', \'mRNA\', \'contig\') of the query. It must be a valid Sequence Ontology term.')
        parser.add_argument('--query-uniquename', action='store_true', help='Use this if the --re-name regular expression matches unique names instead of names in the database.')
        parser.add_argument('--method', choices=['add', 'remove'], default='add', help='Import method')
        parser.add_argument('--re-name', help='Regular expression to extract the feature name from GAF file.')

        args = parser.parse_args(args)

        ti = TripalInstance(**vars(args))

        params = ti.analysis.getBasePayload(args)

        methods = {
            'add': 1,
            'remove': 2,
        }

        params.update({
            'type': 'chado_analysis_go',
            'gaf_file': args.gaf,
            'gaf_file_ext': args.gaf_ext,
            'seq_type': args.query_type,
            'query_uniquename': args.query_uniquename,
            'method': methods[args.method],
            're_name': args.re_name,
            'gojob': 1,
        })

        res = ti.analysis.addAnalysis(params)

        print "New GO analysis created with ID: %s" % res['nid']



#!/usr/bin/env python
import os
import json
import argparse
from tripal import TripalAuth, TripalInstance

class create_organism(object):

    def run(self, args):
        parser = argparse.ArgumentParser(prog=('tripal %s' % self.__class__.__name__), description='Creates an organism into Tripal')
        TripalAuth(parser)
        parser.add_argument("--genus", required=True, help="The genus of the organism")
        parser.add_argument("--species", help="The species of the organism")
        parser.add_argument("--abbr", required=True, help="The abbreviation of the organism")
        parser.add_argument("--common", required=True, help="The common name of the organism")
        parser.add_argument("--description", help="The description of the organism")
        parser.add_argument("--infraspecific-rank", type=int, help="The type id of infraspecific name for any taxon below the rank of species (requires --infraspecific-name.") # TODO use readable names instead of id
        parser.add_argument("--infraspecific-name", help="The infraspecific name for this organism (requires --infraspecific-rank).")

        args = parser.parse_args(args)

        if (args.infraspecific_rank or args.infraspecific_name) and not (args.infraspecific_name and args.infraspecific_rank):
            raise Exception("You should specific both --infraspecific-rank and --infraspecific-name, or none of them.")

        ti = TripalInstance(**vars(args))

        params = {
            'type': 'chado_organism',
            'genus': args.genus,
            'species': args.species,
            'abbreviation': args.abbr,
            'common_name': args.common,
            'description': args.description,
            'type_id': 0,
            'infraspecific_name': '',
        }

        if args.infraspecific_rank:
            params['infraspecific_rank'] = args.infraspecific_rank
            params['infraspecific_name'] = args.infraspecific_name

        res = ti.organism.addOrganism(params)

        print "New organism created with Node ID: %s" % res['nid']



#!/usr/bin/env python
import json
import argparse
from collections import OrderedDict
from tripal import TripalAuth, TripalInstance

class sync_features(object):

    def run(self, args):
        parser = argparse.ArgumentParser(prog=('tripal %s' % self.__class__.__name__), description='Synchronize features')
        TripalAuth(parser)
        parser.add_argument('--job-name', help='Name of the job (default=\'Sync Features\')')
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument('--organism', help='Organism abbreviation or common name')
        group.add_argument('--organism-id', help='Organism ID')
        parser.add_argument('--max-sync', type=int, help='Maximum number of features to sync (default: all)')
        parser.add_argument('--types', nargs='*', help='Space-delimited list of types of records to be synced (e.g. gene mRNA, default: all)')
        parser.add_argument('--ids', nargs='*', help='Space-delimited list of names of records to be synced (e.g. gene0001, default: all)')

        args = parser.parse_args(args)

        ti = TripalInstance(**vars(args))

        org_id = None
        if args.organism:
            org_id = ti.organism.getOrganismByName(args.organism)['organism_id']
        elif args.organism_id:
            org_id = args.organism_id
        else:
            raise Exception("Either --organism or --organism-id is required")

        job_name = args.job_name
        if not job_name:
            job_name = 'Sync Features'

        transaction = 1 # use transaction or not, no reason to disable this

        job_args = OrderedDict()
        job_args['base_table'] = 'feature'
        job_args['max_sync'] = args.max_sync
        job_args['organism_id'] = org_id
        job_args['types'] = args.types
        job_args['ids'] = args.ids
        job_args['linking_table'] = 'chado_feature'
        job_args['node_type'] = 'chado_feature'

        r = ti.jobs.addJob(job_name, 'chado_feature', 'chado_node_sync_records', job_args)
        print 'Sync features job scheduled with id %s' % r['job_id']



from os.path import dirname
import sys, pkgutil

for importer, package_name, _ in pkgutil.iter_modules([dirname(__file__)]):
    full_package_name = '%s.%s' % ('tripal.actions', package_name)
    if full_package_name not in sys.modules:
        module = importer.find_module(package_name).load_module(full_package_name)
